# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
# --------------------------------------
# Copyright (C) 2006-2009 Nicolas Schulz
#               2008-2009 Florian Noeding (Python wrapper)
#
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# *************************************************************************************************

from ctypes import *

if not 'c_bool' in globals():
	c_bool = c_int


__all__ = []

import utils
__all__.append('utils')



try:
	h3d = cdll.LoadLibrary('libHorde3D.so')
except OSError:
	h3d = cdll.LoadLibrary('Horde3D')


RootNode = 1
__all__.append('RootNode')


class EngineOptions(object):
	MaxLogLevel = 1
	MaxNumMessages = 2
	TrilinearFilterung = 3
	AnisotropyFactor = 4
	TexCompression = 5
	LoadTextures = 6
	FastAnimation = 7
	ShadowMapSize = 8
	SampleCount = 9
	WireframeMode = 10
	DebugViewMode = 11
	DumpFailedShaders = 12
__all__.append('EngineOptions')


class ResourceTypes(object):
	Undefined = 0
	SceneGraph = 1
	Geometry = 2
	Animation = 3
	Material = 4
	Code = 5
	Shader = 6
	Texture2D = 7
	TextureCube = 8
	Effect = 9
	Pipeline = 10
__all__.append('ResourceTypes')


class SceneNodeTypes(object):
	Undefined = 0
	Group = 1
	Model = 2
	Mesh = 2
	Joint = 3
	Light = 4
	Camera = 5
	Emitter = 6
__all__.append('SceneNodeTypes')


class SceneNodeParams(object):
	Name = 1
	AttachmentString = 2
__all__.append('SceneNodeParams')


class LightNodeParams(object):
	MaterialRes = 500
	Radius = 501
	FOV = 502
	Col_R = 503
	Col_G = 504
	Col_B = 505
	ShadowMapCount = 506
	ShadowSplitLambda = 507
	ShadowMapBias = 508
__all__.append('LightNodeParams')


class CameraNodeParams(object):
	PipelineRes = 600
	OutputTex = 601
	OutputBufferIndex = 602
	LeftPlane = 603
	RightPlane = 604
	BottomPlane = 605
	TopPlane = 606
	NearPlane = 607
	FarPlane = 608
	Orthographic = 609
	OcclusionCulling = 610
__all__.append('CameraNodeParams')



getVersionString = h3d.getVersionString
getVersionString.restype = c_char_p
getVersionString.argtypes = []
__all__.append('getVersionString')


checkExtension = h3d.checkExtension
checkExtension.restype = c_bool
checkExtension.argtypes = [c_char_p]
__all__.append('checkExtension')


init = h3d.init
init.restype = c_bool
init.argtypes = []
__all__.append('init')


release = h3d.release
release.restype = None
release.argtypes = []
__all__.append('release')


setupViewport = h3d.setupViewport
setupViewport.restype = None
setupViewport.argtypes = [c_int, c_int, c_int, c_int, c_bool]
__all__.append('setupViewport')


render = h3d.render
render.restype = c_bool
render.argtypes = [c_int]
__all__.append('render')


finalizeFrame = h3d.finalizeFrame
finalizeFrame.restype = c_bool
finalizeFrame.argtypes = []
__all__.append('finalizeFrame')


clear = h3d.clear
clear.restype = None
clear.argtypes = []
__all__.append('clear')


_getMessage = h3d.getMessage
_getMessage.restype = c_char_p
_getMessage.argtypes = [POINTER(c_int), POINTER(c_float)]
def getMessage():
	i = c_int()
	f = c_float()
	s = _getMessage(byref(i), byref(f))

	return (s, i.value, f.value)
__all__.append('getMessage')


getOption = h3d.getOption
getOption.restype = c_float
getOption.argtypes = [c_int]
__all__.append('getOption')


_setOption = h3d.setOption
_setOption.restype = c_bool
_setOption.argtypes = [c_int, c_float]
def setOption(param, value):
	return _setOption(param, c_float(value))
__all__.append('setOption')


# getStat
_showOverlay = h3d.showOverlay
_showOverlay.restype = None
_showOverlay.argtypes = [
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_int, c_int]
def showOverlay(
		x_ll, y_ll, u_ll, v_ll,
		x_lr, y_lr, u_lr, v_lr,
		x_ur, y_ur, u_ur, v_ur,
		x_ul, y_ul, u_ul, v_ul,
		layer, materialRes):
	return _showOverlay(
			c_float(x_ll), c_float(y_ll), c_float(u_ll), c_float(v_ll),
			c_float(x_lr), c_float(y_lr), c_float(u_lr), c_float(v_lr),
			c_float(x_ur), c_float(y_ur), c_float(u_ur), c_float(v_ur),
			c_float(x_ul), c_float(y_ul), c_float(u_ul), c_float(v_ul),
			layer, materialRes
			)
__all__.append('showOverlay')


clearOverlays = h3d.clearOverlays
clearOverlays.restype = None
clearOverlays.argtypes = []
__all__.append('clearOverlays')

# getResourceType
# getResourceName
# getNextResource
findResource = h3d.findResource
findResource.restype = c_int
findResource.argtypes = [c_int, c_char_p]
__all__.append('findResource')


addResource = h3d.addResource
addResource.restype = c_int
addResource.argtypes = [c_int, c_char_p, c_int]
__all__.append('addResource')



# cloneResource
# removeResource
# isResourceLoaded
# loadResource
# unloadResource
# getResourceParami
# setResourceParami
# getResourceParamf
# setResourceParamf
# getResourceParamstr
# setResourceParamstr
# getResourceData
# updateResourceData
# queryUnloadedResource
# releaseUnusedResources
# createTexture2D
# setShaderPreambles
_setMaterialUniform = h3d.setMaterialUniform
_setMaterialUniform.restype = c_bool
_setMaterialUniform.argtypes = [c_int, c_char_p, c_float, c_float, c_float, c_float]
def setMaterialUniform(materialRes, name, a, b, c, d):
	return _setMaterialUniform(materialRes, name, c_float(a), c_float(b), c_float(c), c_float(d))
__all__.append('setMaterialUniform')


# setPipelineStageActivation
# getPipelineStageActivation
# getNodeType
# getNodeParent
# setNodeParent
# getNodeChild

addNodes = h3d.addNodes
addNodes.restype = c_int
addNodes.argtypes = [c_int, c_int]
__all__.append('addNodes')


# removeNode
# setNodeActivation
# checkNodeTransformFlag
# getNodeTransform


_setNodeTransform = h3d.setNodeTransform
_setNodeTransform.restype = c_bool
_setNodeTransform.argtypes = [c_int,
					c_float, c_float, c_float,
					c_float, c_float, c_float,
					c_float, c_float, c_float,
					]
def setNodeTransform(node, tx, ty, tz, rx, ry, rz, sx, sy, sz):
	return _setNodeTransform(node,
			c_float(tx), c_float(ty), c_float(tz),
			c_float(rx), c_float(ry), c_float(rz),
			c_float(sx), c_float(sy), c_float(sz),
			)
__all__.append('setNodeTransform')





# getNodeTransformMatrices
# setNodeTransformMatrices
_getNodeParamf = h3d.getNodeParamf
_getNodeParamf.restype = c_float
_getNodeParamf.argtypes = [c_int, c_int]
def getNodeParamf(node, param):
	return _getNodeParamf(node, param).value
__all__.append('getNodeParamf')


_setNodeParamf = h3d.setNodeParamf
_setNodeParamf.restype = c_bool
_setNodeParamf.argtypes = [c_int, c_int, c_float]
def setNodeParamf(node, param, value):
	return _setNodeParamf(node, param, c_float(value))
__all__.append('setNodeParamf')


getNodeParami = h3d.getNodeParami
getNodeParami.restype = c_int
getNodeParami.argtypes = [c_int, c_int]
__all__.append('getNodeParami')

setNodeParami = h3d.setNodeParami
setNodeParami.restype = c_bool
setNodeParami.argtypes = [c_int, c_int, c_int]
__all__.append('setNodeParami')


# getNodeParamstr
# setNodeParamstr
# getNodeAABB
findNodes = h3d.findNodes
findNodes.restype = c_int
findNodes.argtypes = [c_int, c_char_p, c_int]
__all__.append('findNodes')


getNodeFindResult = h3d.getNodeFindResult
getNodeFindResult.restype = c_int
getNodeFindResult.argtypes = [c_int]
__all__.append('getNodeFindResult')


# castRay
# getCastRayResult
# checkNodeVisibility
# addGroupNode
# addModelNode


setupModelAnimStage = h3d.setupModelAnimStage
setupModelAnimStage.restype = c_bool
setupModelAnimStage.argtypes = [c_int, c_int, c_int, c_char_p, c_bool]
__all__.append('setupModelAnimStage')


_setModelAnimParams = h3d.setModelAnimParams
_setModelAnimParams.restype = c_bool
_setModelAnimParams.argtypes = [c_int, c_int, c_float, c_float]
def setModelAnimParams(modelNode, stage, time, weight):
	return _setModelAnimParams(modelNode, stage, c_float(time), c_float(weight))
__all__.append('setModelAnimParams')



# setModelMorper
# addMeshNode
# addJointNode
addLightNode = h3d.addLightNode
addLightNode.restype = c_int
addLightNode.argtypes = [c_int, c_char_p, c_int, c_char_p, c_char_p]
__all__.append('addLightNode')


setLightContexts = h3d.setLightContexts
setLightContexts.restype = c_bool
setLightContexts.argtypes = [c_int, c_char_p, c_char_p]
__all__.append('setLightContexts')


addCameraNode = h3d.addCameraNode
addCameraNode.restype = c_int
addCameraNode.argtypes = [c_int, c_char_p, c_int]
__all__.append('addCameraNode')


_setupCameraView = h3d.setupCameraView
_setupCameraView.restype = c_bool
_setupCameraView.argtypes = [c_int, c_float, c_float, c_float, c_float]
def setupCameraView(cameraNode, fov, aspect, nearDist, farDist):
	return _setupCameraView(cameraNode, c_float(fov), c_float(aspect), c_float(nearDist), c_float(farDist))
__all__.append('setupCameraView')




